from functools import wraps
from typing import (
    Coroutine,
    Awaitable,
    Callable,
    TypeVar,
    Any,
)

from lanyard.http.exception import UnauthorizedError
from lanyard.exception import NoPermissionError

R = TypeVar("R")


def handle_error(
    func: Callable[..., Awaitable[R]],
) -> Callable[..., Coroutine[Any, Any, R]]:
    """
    A decorator that handles certain HTTP errors and converts them into API Wrapper exceptions.

    :param func: Async method to be decorated

    :raise NoPermissionError: When the request returns ``Unauthorized status (401)``

    :return: Decorated function with error handling
    """

    @wraps(func)
    async def wrapper(*args: Any, **kwargs: Any) -> R:
        try:
            return await func(*args, **kwargs)
        except UnauthorizedError:
            raise NoPermissionError()

    return wrapper


__all__ = ["handle_error"]



from typing import Optional, Dict
from urllib import parse
from pydantic import BaseModel, Field, field_validator

from .constant import API_BASE_URL, API_DEFAULT_VERSION
from .exception import UnknownAPIVersionError


class LanyardConfig(BaseModel):
    """
    Lanyard API client configuration.

    This class contains settings for connecting to the Lanyard API and making management requests.
    """

    base_url: str = Field(default=API_BASE_URL)
    api_version: int = Field(default=API_DEFAULT_VERSION)
    raise_for_status: bool = Field(default=True)
    token: Optional[str] = Field(default=None)
    timeout: Optional[float] = Field(default=None)

    @field_validator("api_version", mode="before")
    @classmethod
    def _validate_api_version(cls, v: int) -> int:
        """
        Validator for converting a dictionary to a list of KeyValueData.

        :param v: Value to convert.

        :return: List of ``KeyValueData``.
        """

        if v is not None and v <= 0:
            raise UnknownAPIVersionError(v)

        return v

    @property
    def url(self) -> str:
        """
        Parses the underlying HTTP link and API version

        :return: HTTP link
        """

        return parse.urljoin(base=self.base_url, url=f"v{self.api_version}/")

    @property
    def headers(self) -> Dict[str, str]:
        """
        Returns a dictionary specifying the content type and user token.

        :return: Return the finished dictionary
        """

        headers: Dict[str, str] = {"Content-Type": "application/json"}

        if self.token is not None:
            headers["Authorization"] = self.token

        return headers


__all__ = ["LanyardConfig"]



import json

from typing import Optional, Dict, Any

from lanyard.http.exception import NotFoundError
from lanyard.exception import UserNotMonitoredError
from lanyard.decorator import requires_token, handle_error
from lanyard.http import HTTPResponse
from lanyard.model import Response, ResponseData

from .base import BaseService


class UserService(BaseService):
    @handle_error
    async def by_id(self, user_id: int) -> Optional[ResponseData]:
        try:
            response: HTTPResponse = await self._context.http.get(
                path=f"users/{user_id}",
                headers=self._context.config.headers,
            )

            response_json: Dict[str, Any] = json.loads(response.content)
            response_model: Response = Response(**response_json)

            if response_model.data is not None:
                return response_model.data

            return None
        except NotFoundError:
            raise UserNotMonitoredError()

    @requires_token
    @handle_error
    async def me(self) -> Optional[ResponseData]:
        try:
            response: HTTPResponse = await self._context.http.get(
                path="users/@me",
                headers=self._context.config.headers,
            )

            response_json: Dict[str, Any] = json.loads(response.content)
            response_model: Response = Response(**response_json)

            if response_model.data is not None:
                return response_model.data

            return None
        except NotFoundError:
            raise UserNotMonitoredError()


__all__ = ["UserService"]



from abc import ABC, abstractmethod
from urllib import parse
from typing import Optional, Any

from .type import Headers, Params, Data
from .decorator import handle_http_error
from .response import HTTPResponse


class BaseHTTP(ABC):
    def __init__(
        self,
        base_url: str,
        raise_for_status: bool = True,
        timeout: Optional[float] = None,
    ) -> None:
        """
        The base abstract class for all HTTP-related classes.

        :param base_url: Base URL of HTTP response (example: ``https://api.lanyard.rest/``).
        :param raise_for_status: Raise HTTP status code of HTTP response (example: ``True``).
        :param timeout: Timeout for HTTP requests (example: ``None``).
        """

        self.base_url: str = base_url
        self.raise_for_status: bool = raise_for_status
        self.timeout: Optional[float] = timeout

    @staticmethod
    def _build_url(base_url: str, path: Optional[str]) -> str:
        if path is None:
            return base_url

        return parse.urljoin(base_url, path)

    @abstractmethod
    async def _request(self, method: str, url: str, **kwargs: Any) -> HTTPResponse:
        raise NotImplementedError()

    @handle_http_error
    async def get(
        self,
        path: Optional[str] = None,
        params: Optional[Params] = None,
        headers: Optional[Headers] = None,
        **kwargs: Any,
    ) -> HTTPResponse:
        """
        Executes an async HTTP GET request.

        :param path: URL for request.
        :param params: Query parameters to include in the URL.
        :param headers: Headers to include in the request.
        :param kwargs: Additional fields for HTTP client attributes.
        """

        url: str = self._build_url(
            base_url=self.base_url,
            path=path,
        )

        return await self._request(
            method="GET",
            url=url,
            params=params,
            headers=headers,
            **kwargs,
        )

    @handle_http_error
    async def put(
        self,
        path: Optional[str] = None,
        data: Optional[Data] = None,
        params: Optional[Params] = None,
        headers: Optional[Headers] = None,
        **kwargs: Any,
    ) -> HTTPResponse:
        """
        Executes an async HTTP PUT request.

        :param path: URL for request.
        :param data: Body of request to include in the request.
        :param params: Query parameters to include in the URL.
        :param headers: Headers to include in the request.
        :param kwargs: Additional fields for HTTP client attributes.
        """

        url: str = self._build_url(
            base_url=self.base_url,
            path=path,
        )

        return await self._request(
            method="PUT",
            url=url,
            data=data,
            params=params,
            headers=headers,
            **kwargs,
        )

    @handle_http_error
    async def patch(
        self,
        path: Optional[str] = None,
        data: Optional[Data] = None,
        params: Optional[Params] = None,
        headers: Optional[Headers] = None,
        **kwargs: Any,
    ) -> HTTPResponse:
        """
        Executes an async HTTP PATCH request.

        :param path: URL for request.
        :param data: Body of request to include in the request.
        :param params: Query parameters to include in the URL.
        :param headers: Headers to include in the request.
        :param kwargs: Additional fields for HTTP client attributes.
        """

        url: str = self._build_url(
            base_url=self.base_url,
            path=path,
        )

        return await self._request(
            method="PATCH",
            url=url,
            data=data,
            params=params,
            headers=headers,
            **kwargs,
        )

    @handle_http_error
    async def delete(
        self,
        path: Optional[str] = None,
        params: Optional[Params] = None,
        headers: Optional[Headers] = None,
        **kwargs: Any,
    ) -> HTTPResponse:
        """
        Executes an async HTTP DELETE request.

        :param path: URL for request.
        :param params: Query parameters to include in the URL.
        :param headers: Headers to include in the request.
        :param kwargs: Additional fields for HTTP client attributes.
        """

        url: str = self._build_url(
            base_url=self.base_url,
            path=path,
        )

        return await self._request(
            method="DELETE",
            url=url,
            params=params,
            headers=headers,
            **kwargs,
        )


__all__ = ["BaseHTTP"]



from .context import ServiceContext
from .base import BaseService
from .user import UserService
from .kv import KVService

__all__ = [
    "ServiceContext",
    "BaseService",
    "UserService",
    "KVService",
]



from functools import wraps
from typing import (
    TYPE_CHECKING,
    Coroutine,
    Awaitable,
    Callable,
    Optional,
    Final,
    Type,
    Dict,
    Any,
)

from lanyard.http.exception import (
    HTTPResponseError,
    HTTPClientError,
    HTTPServerError,
    NotFoundError,
    UnauthorizedError,
)
from lanyard.http.response import HTTPResponse

if TYPE_CHECKING:
    from lanyard.http.base import BaseHTTP

_HTTP_ERROR_MAP: Final[Dict[int, Type[HTTPResponseError]]] = {
    401: UnauthorizedError,
    404: NotFoundError,
}


def handle_http_error(
    func: Callable[..., Awaitable[HTTPResponse]],
) -> Callable[..., Coroutine[Any, Any, HTTPResponse]]:
    """
    Decorator for handling HTTP errors in async methods.

    Works when the ``raise_for_status`` attribute is ``True``.

    :param func: An async method returning an HTTPResponse

    :raise HTTPResponseError: If an unknown HTTP error occurs
    :raise HTTPClientError: If an unknown HTTP client error occurs (4xx)
    :raise HTTPServerError: If an unknown HTTP server error occurs (5xx)
    :raise NotFoundError: If an HTTP error with code 404 occurs
    :raise UnauthorizedError: If an HTTP error with code 401 occurs

    :return: A decorated function that handles HTTP errors
    """

    @wraps(func)
    async def wrapper(self: "BaseHTTP", *args: Any, **kwargs: Any) -> HTTPResponse:
        if not getattr(self, "raise_for_status") or not self.raise_for_status:
            return await func(self, *args, **kwargs)

        response: HTTPResponse = await func(self, *args, **kwargs)

        status_code: int = response.status_code

        if 200 <= status_code <= 299:
            return response

        exception_class: Optional[Type[HTTPResponseError]] = _HTTP_ERROR_MAP.get(
            status_code
        )
        if exception_class is not None:
            raise exception_class(**response.model_dump())

        match status_code:
            case status_code if 400 <= status_code <= 499:
                raise HTTPClientError(**response.model_dump())
            case status_code if 500 <= status_code <= 599:
                raise HTTPServerError(**response.model_dump())
            case _:
                raise HTTPResponseError(**response.model_dump())

    return wrapper


__all__ = ["handle_http_error"]



from functools import wraps
from typing import (
    TYPE_CHECKING,
    Coroutine,
    Awaitable,
    Callable,
    TypeVar,
    Any,
)

from lanyard.exception import TokenRequiredError

if TYPE_CHECKING:
    from lanyard.service import BaseService

R = TypeVar("R")


def requires_token(
    func: Callable[..., Awaitable[R]],
) -> Callable[..., Coroutine[Any, Any, R]]:
    """
    A decorator that ensures that a token is present before making an httpx request.

    Throws an exception when the ``token`` attribute is missing or is ``False``.

    :param func: Async method

    :raise TokenRequiredError: Token is missing

    :return: Decorated function that checks for the presence of a token
    """

    @wraps(func)
    async def wrapper(self: "BaseService", *args: Any, **kwargs: Any) -> R:
        if not getattr(self, "_context") or self._context.config.token is None:
            raise TokenRequiredError()

        return await func(self, *args, **kwargs)

    return wrapper


__all__ = ["requires_token"]



from typing import Any


class HTTPError(Exception):
    def __init__(
        self,
        method: str,
        url: str,
        message: str,
    ) -> None:
        """
        Base exception for all HTTP-related errors.

        :param method: Method of the request.
        :param url: URL of the request.
        :param message: Error message.
        """

        self.method: str = method
        self.url: str = url
        self.message: str = message

        super().__init__(f"{self.method} {self.url} returned: {self.message}")


class HTTPResponseError(Exception):
    def __init__(
        self,
        method: str,
        url: str,
        status_code: int,
        content: Any,
    ) -> None:
        """
        Base exception for all HTTP-related response errors.

        :param method: Method of HTTP response.
        :param url: URL of HTTP response.
        :param status_code: HTTP status code of HTTP response.
        :param content: Content of HTTP response.
        """

        self.method: str = method
        self.url: str = url
        self.status_code: int = status_code
        self.content: Any = content

        super().__init__(
            f"{self.method} {self.url} returned {self.status_code}: {self.content}"
        )


class HTTPClientError(HTTPResponseError):
    """
    Exception for HTTP related client errors (4xx).
    """

    ...


class HTTPServerError(HTTPResponseError):
    """
    Exception for HTTP related server errors (5xx).
    """

    ...


class UnauthorizedError(HTTPClientError):
    """
    Exception for HTTP client error related to "unauthorized" (401) error.
    """

    ...


class NotFoundError(HTTPClientError):
    """
    Exception for HTTP client error related to "not found" (404) error.
    """

    ...


__all__ = [
    "HTTPError",
    "HTTPResponseError",
    "HTTPClientError",
    "HTTPServerError",
    "UnauthorizedError",
    "NotFoundError",
]



class HTTPXError(Exception):
    def __init__(
        self,
        message: str,
    ) -> None:
        """
        A base exception for all errors related to the HTTPX library-based client.

        :param message: Error message.
        """

        self.message: str = message

        super().__init__(self.message)


class ContextRequiredError(HTTPXError):
    def __init__(self) -> None:
        """
        Exception when it is necessary to use context.
        """

        super().__init__("Use an asynchronous context")


__all__ = [
    "HTTPXError",
    "ContextRequiredError",
]



from pydantic import BaseModel, Field, field_validator
from datetime import datetime
from typing import Optional, List, Dict, Any

from .type import Snowflake
from .enum import DiscordStatus


class AvatarDecorationData(BaseModel):
    """
    Data model for decorating the avatar of a Discord user.
    """

    asset: str = Field(...)
    sku_id: Snowflake = Field(...)


class PrimaryGuildData(BaseModel):
    """
    Data model of the user's main server (guild).
    """

    identity_guild_id: Optional[Snowflake] = Field(default=None)
    identity_enabled: Optional[bool] = Field(default=None)
    tag: Optional[str] = Field(default=None)
    badge: Optional[str] = Field(default=None)


class DiscordUser(BaseModel):
    """
    The basic user model of Discord.
    """

    id: Snowflake = Field(...)
    username: str = Field(...)
    discriminator: str = Field(...)
    bot: bool = Field(...)
    public_flags: int = Field(...)
    global_name: Optional[str] = Field(default=None)
    avatar: Optional[str] = Field(default=None)
    avatar_decoration_data: Optional[AvatarDecorationData] = Field(default=None)
    primary_guild: Optional[PrimaryGuildData] = Field(default=None)


class KeyValueData(BaseModel):
    """
    A model for storing key-value pairs.
    """

    key: str = Field(...)
    value: str = Field(...)


class TimestampData(BaseModel):
    """
    Timestamp model.
    """

    start: Optional[datetime] = Field(default=None)
    end: Optional[datetime] = Field(default=None)


class AssetData(BaseModel):
    """
    Activity resource (image) data model.
    """

    large_image: Optional[str] = Field(default=None)
    large_text: Optional[str] = Field(default=None)
    large_url: Optional[str] = Field(default=None)
    small_image: Optional[str] = Field(default=None)
    small_text: Optional[str] = Field(default=None)
    small_url: Optional[str] = Field(default=None)


class ActivityData(BaseModel):
    """
    Discord user activity data model.
    """

    id: Snowflake = Field(...)
    type: int = Field(...)
    created_at: datetime = Field(...)
    application_id: Optional[Snowflake] = Field(default=None)
    sync_id: Optional[Snowflake] = Field(default=None)
    session_id: Optional[Snowflake] = Field(default=None)
    flags: Optional[int] = Field(default=None)
    state: Optional[str] = Field(default=None)
    details: Optional[str] = Field(default=None)
    timestamps: Optional[TimestampData] = Field(default=None)
    assets: Optional[AssetData] = Field(default=None)


class SpotifyData(BaseModel):
    """
    Spotify Activity Data Model.
    """

    timestamps: Optional[TimestampData] = Field(default=None)
    album: Optional[str] = Field(default=None)
    album_art_url: Optional[str] = Field(default=None)
    artist: Optional[str] = Field(default=None)
    song: Optional[str] = Field(default=None)
    track_id: Optional[Snowflake] = Field(default=None)


class ResponseData(BaseModel):
    """
    The basic data model of a response with user status information.
    """

    discord_user: DiscordUser = Field(...)
    kv: List[KeyValueData] = Field(...)
    listening_to_spotify: bool = Field(...)
    active_on_discord_embedded: bool = Field(...)
    active_on_discord_mobile: bool = Field(...)
    active_on_discord_desktop: bool = Field(...)
    active_on_discord_web: bool = Field(...)
    activities: List[ActivityData] = Field(...)
    discord_status: DiscordStatus = Field(...)
    spotify: Optional[SpotifyData] = Field(default=None)

    @field_validator("kv", mode="before")
    @classmethod
    def _validate_kv(cls, v: Dict[str, Any]) -> List[KeyValueData]:
        """
        Validator for converting a dictionary to a list of KeyValueData.

        :param v: Value to convert.
        :return: List of ``KeyValueData``.
        """

        return [KeyValueData(key=key, value=value) for key, value in v.items()]


class ResponseError(BaseModel):
    """
    Response error model.
    """

    code: str = Field(...)
    message: str = Field(...)


class Response(BaseModel):
    """
    Basic API response model.
    """

    success: bool = Field(...)
    data: Optional[ResponseData] = Field(default=None)
    error: Optional[ResponseError] = Field(default=None)


__all__ = [
    "Response",
    "ResponseData",
    "ResponseError",
]



from typing import Final

API_BASE_URL: Final[str] = "https://api.lanyard.rest/"
"""Base URL for Lanyard API"""

API_DEFAULT_VERSION: Final[int] = 1
"""Default API version"""

__all__ = [
    "API_BASE_URL",
    "API_DEFAULT_VERSION",
]



from __future__ import annotations

from typing import TYPE_CHECKING, Any
from pydantic import BaseModel, Field

if TYPE_CHECKING:
    from lanyard.http import BaseHTTP
    from lanyard.rest import LanyardConfig


class ServiceContext(BaseModel):
    http: Any = Field(...)
    config: Any = Field(...)

    def __init__(self, http: BaseHTTP, config: LanyardConfig):
        super().__init__(http=http, config=config)


__all__ = ["ServiceContext"]



from types import TracebackType
from typing import Optional, Type, Self

from .service import ServiceContext, UserService, KVService
from .config import LanyardConfig
from .http import BaseHTTP, HTTPX


class LanyardClient:
    def __init__(
        self,
        http: Optional[BaseHTTP] = None,
        config: Optional[LanyardConfig] = None,
    ) -> None:
        """
        The main client class for working with the Lanyard API, implementing an asynchronous context manager.

        :param http: HTTP client for making requests
        :param config: Client configuration
        """

        self.config: LanyardConfig = self._build_config(
            config=config,
        )

        self._http: BaseHTTP = self._build_http_client(
            http=http,
            base_url=self.config.url,
            timeout=self.config.timeout,
            raise_for_status=self.config.raise_for_status,
        )

        self._context: ServiceContext = ServiceContext(
            config=self.config,
            http=self._http,
        )
        self.user: UserService = UserService(
            context=self._context,
        )
        self.kv: KVService = KVService(
            context=self._context,
        )

    @staticmethod
    def _build_config(
        config: Optional[LanyardConfig] = None,
    ) -> LanyardConfig:
        if config is None:
            return LanyardConfig()

        return config

    @staticmethod
    def _build_http_client(
        base_url: str,
        raise_for_status: bool = True,
        http: Optional[BaseHTTP] = None,
        timeout: Optional[float] = None,
    ) -> BaseHTTP:
        if http is not None:
            return http

        return HTTPX(
            base_url=base_url,
            timeout=timeout,
            raise_for_status=raise_for_status,
        )

    async def __aenter__(self) -> Self:
        if hasattr(self._http, "__aenter__"):
            await self._http.__aenter__()

        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        if hasattr(self._http, "__aexit__"):
            await self._http.__aexit__(exc_type, exc_val, exc_tb)


__all__ = ["LanyardClient"]



import pytest

from typing import Optional

from lanyard import LanyardConfig
from lanyard.constant import API_BASE_URL, API_DEFAULT_VERSION
from lanyard.exception import UnknownAPIVersionError


class TestLanyardConfig:
    def test_init(self) -> None:
        config: LanyardConfig = LanyardConfig()

        assert config.base_url == API_BASE_URL
        assert config.api_version == API_DEFAULT_VERSION
        assert config.url == "https://api.lanyard.rest/v1/"
        assert config.headers == {"Content-Type": "application/json"}
        assert config.token is None
        assert config.timeout is None
        assert config.raise_for_status is True

    def test_custom_init(self) -> None:
        base_url: str = "https://custom.example.com/"
        api_version: int = 2
        token: str = "test_token_123"
        timeout: float = 30.0
        raise_for_status: bool = True

        config: LanyardConfig = LanyardConfig(
            base_url=base_url,
            api_version=api_version,
            token=token,
            timeout=timeout,
            raise_for_status=raise_for_status,
        )

        assert config.base_url == base_url
        assert config.api_version == api_version
        assert config.url == "https://custom.example.com/v2/"
        assert config.headers == {
            "Content-Type": "application/json",
            "Authorization": token,
        }
        assert config.token == token
        assert config.timeout == timeout
        assert config.raise_for_status is raise_for_status

    @pytest.mark.parametrize("token", [None, "xxxxxe47b4dedc260f13a0ec57d78745", ""])
    def test_headers_param(self, token: Optional[str]) -> None:
        config: LanyardConfig = LanyardConfig(token=token)

        if token is None:
            assert config.headers == {"Content-Type": "application/json"}
        else:
            assert config.headers == {
                "Content-Type": "application/json",
                "Authorization": token,
            }

    @pytest.mark.parametrize(
        "api_version, should_raise",
        [
            (1, False),
            (2, False),
            (0, True),
            (-1, True),
        ],
    )
    def test_api_version_param(self, api_version: int, should_raise: bool) -> None:
        if should_raise:
            with pytest.raises(UnknownAPIVersionError):
                LanyardConfig(api_version=api_version)
        else:
            config: LanyardConfig = LanyardConfig(api_version=api_version)
            assert config.api_version == api_version



from pydantic import BaseModel, Field


class HTTPResponse(BaseModel):
    """
    Model for a standardized HTTP response.
    """

    method: str = Field(...)
    url: str = Field(...)
    status_code: int = Field(...)
    content: bytes = Field(...)


__all__ = ["HTTPResponse"]



from types import TracebackType
from typing import Optional, Self, Type, Any

from httpx import AsyncClient, Response

from lanyard.http.base import BaseHTTP
from lanyard.http.response import HTTPResponse
from lanyard.http.exception import HTTPError

from .exception import ContextRequiredError


# TODO: Requires write description for this code


class HTTPX(BaseHTTP):
    def __init__(
        self,
        base_url: str,
        raise_for_status: bool = True,
        timeout: Optional[float] = None,
    ) -> None:
        self.base_url: str = base_url
        self.raise_for_status: bool = raise_for_status
        self.timeout: Optional[float] = timeout

        self._session: Optional[AsyncClient] = None

        super().__init__(
            base_url=self.base_url,
            raise_for_status=self.raise_for_status,
            timeout=self.timeout,
        )

    async def __aenter__(self) -> Self:
        if self._session is None:
            self._session = AsyncClient(
                base_url=self.base_url,
                timeout=self.timeout,
            )

        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        if self._session is not None:
            await self._session.aclose()

    @property
    def session(self) -> AsyncClient:
        if not self._session:
            raise ContextRequiredError()

        return self._session

    async def _request(
        self,
        method: str,
        url: str,
        **kwargs: Any,
    ) -> HTTPResponse:
        try:
            response: Response = await self.session.request(
                method=method,
                url=url,
                **kwargs,
            )

            return HTTPResponse(
                method=method,
                url=url,
                status_code=response.status_code,
                content=response.content,
            )
        except Exception as exc:
            raise HTTPError(
                method=method,
                url=url,
                message=str(exc),
            )


__all__ = ["HTTPX"]



class LanyardError(Exception):
    def __init__(
        self,
        message: str,
    ) -> None:
        """
        Base exception for all errors with Lanyard client.

        :param message: Error message.
        """

        self.message: str = message

        super().__init__(self.message)


class KVInvalidError(LanyardError):
    def __init__(self) -> None:
        """
        Exception thrown when the KV body value is not an object (dict)
        """

        super().__init__("Value must be an object (dict)")


class KVValidationError(LanyardError):
    def __init__(self) -> None:
        """
        An exception is thrown when the limits for keys or values in KV are violated.
        """

        super().__init__(
            "Violation of KV limits (https://github.com/Phineas/lanyard?tab=readme-ov-file#limits)"
        )


class NoPermissionError(LanyardError):
    def __init__(self) -> None:
        """
        Exception raised when a user does not have permission to perform the operation.
        """

        super().__init__("You do not have permission to access this resource")


class TokenRequiredError(LanyardError):
    def __init__(self) -> None:
        """
        Exception for methods that require a token to be specified.
        """

        super().__init__("A token must be specified for this request")


class UserNotMonitoredError(LanyardError):
    def __init__(self) -> None:
        """
        Exception when the requested user is not monitored by the service
        """

        super().__init__("User not monitored")


class UnknownAPIVersionError(LanyardError):
    def __init__(self, version: int) -> None:
        """
        Exception for unknown API version

        :param version: API version (default: ``1``).
        """

        self.version: int = version

        super().__init__(f"API version ({self.version}) must be positive integer")


__all__ = [
    "LanyardError",
    "KVInvalidError",
    "NoPermissionError",
    "KVValidationError",
    "TokenRequiredError",
    "UserNotMonitoredError",
    "UnknownAPIVersionError",
]



import json

from typing import Dict, Any

from lanyard.http.exception import NotFoundError
from lanyard.exception import KVValidationError, KVInvalidError
from lanyard.decorator import requires_token, handle_error

from .base import BaseService


class KVService(BaseService):
    @requires_token
    @handle_error
    async def set(self, user_id: int, key: str, value: str) -> None:
        try:
            await self._context.http.put(
                path=f"users/{user_id}/kv/{key}",
                headers=self._context.config.headers,
                data=value,
            )

            return None
        except NotFoundError:
            raise KVValidationError()

    @requires_token
    @handle_error
    async def delete(self, user_id: int, key: str) -> None:
        try:
            await self._context.http.delete(
                path=f"users/{user_id}/kv/{key}",
                headers=self._context.config.headers,
            )

            return None
        except NotFoundError:
            raise KVValidationError()

    @requires_token
    @handle_error
    async def merge(self, user_id: int, value: Dict[str, Any]) -> None:
        if not isinstance(value, dict):
            raise KVInvalidError()

        try:
            await self._context.http.patch(
                path=f"users/{user_id}/kv",
                headers=self._context.config.headers,
                data=json.dumps(value),
            )

            return None
        except NotFoundError:
            raise KVValidationError()


__all__ = ["KVService"]



from abc import ABC

from .context import ServiceContext


class BaseService(ABC):
    def __init__(self, context: ServiceContext):
        self._context: ServiceContext = context


__all__ = ["BaseService"]



from enum import StrEnum


class DiscordStatus(StrEnum):
    """
    Discord user statuses.
    """

    ONLINE = "online"
    OFFLINE = "offline"
    DND = "dnd"
    IDLE = "idle"


__all__ = ["DiscordStatus"]
